#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <termios.h>
#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAIN_C
#include "exitfail.h"

/**** sample: AGRI-RD serial setting: begin ****/
#define SERIAL_BAUDRATE B9600
/**** sample: AGRI-RD serial setting: end ****/
#define BUF_SIZE 2048

static int serial_fd = -1;
static struct termios old_tio;

static int terminated = 0;

#define BASENAME(p) ((strrchr((p), '/') ? : ((p) - 1)) +1)
#define ARRAY_SIZE(a) (sizeof(a) /sizeof((a)[0]))

#define __unused __attribute__((unused))

/**** sample: AGRI-RD response: begin ****/
static unsigned char res_data[] = "Q,"; /* AGRI-RD response: data */
/**** sample: AGRI-RD response: end ****/

static void terminate_sig_handler(__unused int signum)
{
	terminated = 1;
}

static void set_sig_handler(int sig_list[], ssize_t num, __sighandler_t handler)
{
	struct sigaction sa;
	int i;

	memset(&sa, 0, sizeof(sa));
	sa.sa_handler = handler;

	for(i=0; i<num; i++)
		if(sigaction(sig_list[i], &sa, NULL) < 0)
			exitfail_errno("sigaction");
}

static void restore_serial(void)
{
	int ret;

	ret = tcsetattr(serial_fd, TCSANOW, &old_tio);
	if(ret < 0) exitfail_errno("tcsetattr");
}

static void setup_serial(int fd)
{
	struct termios tio;
	int ret;

	ret = tcgetattr(fd, &old_tio);
	if(ret) exitfail_errno("tcsgetattr");

	if(atexit(restore_serial)) exitfail_errno("atexit");

	memset(&tio, 0, sizeof(tio));
	tio.c_iflag = IGNBRK | IGNPAR;
	tio.c_cflag = CS8 | CLOCAL | CREAD;
	tio.c_cc[VTIME] = 0;
	tio.c_cc[VMIN] = 1;
	ret = cfsetspeed(&tio, SERIAL_BAUDRATE);
	if(ret < 0) exitfail_errno("cfsetspeed");

	ret = tcflush(fd, TCIFLUSH);
	if(ret < 0) exitfail_errno("tcflush");

	ret = tcsetattr(fd, TCSANOW, &tio);
	if(ret) exitfail_errno("tcsetattr");
}

int main(int argc, char *argv[])
{
	int terminate_sig_list[] = {
		SIGHUP, SIGINT, SIGQUIT, SIGPIPE, SIGTERM
	};

/**** sample: variable: begin ****/
	ssize_t ret;
       char buf[BUF_SIZE];
       char buf1[BUF_SIZE];
	int i;
	FILE *fp;
	int flag_data, flag_lf;
	int cnt_cmp;
	ssize_t analen;
/**** sample: variable: end ****/

/**** sample: variable for senseor data: begin ****/
	char *tok;
	int tok_cnt;
	int cpt;
	float cpm, rad;
/**** sample: for senseor data: end ****/

	exitfail_init();

	if(argc < 2) {
		printf("Usage: %s <device>\n", BASENAME(argv[0]));
		return EXIT_SUCCESS;
	}

	serial_fd = open(argv[1], O_RDWR | O_NOCTTY);
	if(serial_fd < 0) exitfail_errno("open");

	set_sig_handler(terminate_sig_list, ARRAY_SIZE(terminate_sig_list),
			terminate_sig_handler);

	setup_serial(serial_fd);

/**** sample: AGRI-RD data read: begin ****/
	flag_data = 0;
	flag_lf = 0;
	cnt_cmp = 0;
	analen = 0;

	while(!terminated) {
		ret = read(serial_fd, buf, BUF_SIZE);
		if(ret < 0) exitfail_errno("read");

		for(i = 0; i < ret && flag_lf != 1; i++) {
//			printf("%c", buf[i]);
			if(flag_data != 1 && buf[i] == res_data[cnt_cmp]) {
				cnt_cmp++;
				buf1[analen++] = buf[i];
			}
			else {
				cnt_cmp = 0;
				if(flag_data == 1) buf1[analen++] = buf[i];
				else analen = 0;
			}
			if(cnt_cmp == sizeof(res_data) - 1) flag_data = 1;
			if(flag_data == 1 && buf[i] == '\n') {
				flag_lf = 1;
				buf1[analen] = '\0';
				if(flag_lf == 1) {
/**** <--- Indent Shift: begin ****/

/**** sample: sensor data to log file: begin ****/
fp = fopen("log_rd.txt", "w");
fprintf(fp, "%s", buf1);
fclose(fp);
/**** sample: sensor data to log file: end ****/

/**** sample: execute "sample_action.sh" if it's exist: begin ****/
if((fp = fopen("sample_action.sh", "r")) != NULL) {
	system("./sample_action.sh");
	fclose(fp);
}
/**** sample: execute "sample_action.sh" if it's exist: end ****/

/**** sample: sensor data to console: begin ****/
printf("******************************\n");
printf("%s", buf1);
tok_cnt = 0;
tok = strtok(buf1, ",\n");
while(tok != NULL) {
	switch(tok_cnt) {
	case 0:
		printf("Mark: %s\n", tok);
		break;
	case 1:
		cpt = atoi(tok);
		printf("CPT: %d\n", cpt);
		break;
	case 2:
		cpm = atof(tok);
		printf("CPM: %.1f\n", cpm);
		break;
	case 3:
		rad = atof(tok);
		printf("Radiation: %.3f [uSv/h]\n", rad);
		printf("\n");
		break;
	default:
	break;	
	}
	tok_cnt++;
	tok = strtok(NULL, ",\n");
}
/**** sample: sensor data to console: end ****/

/**** sample: clear analyzing: begin ****/
flag_data = 0;
flag_lf = 0;
cnt_cmp = 0;
analen = 0;
/**** sample: clear analyzing: end ****/
/**** <--- Indent Shift: end ****/
				}
			}
		}
	}
/**** sample: AGRI-RD data request: end ****/

	return EXIT_SUCCESS;
}
